% b1in.m: interior-point solution of b1

clear; clf
xstar=[0;1];               % optimal x for finding error to plot
v=[-2;2;0;0;1;1];          % starting J must be nonsingular
mu=1;                      % starting barrier multiplier
for k=1:52                 % solve a sequence of barrier problems
    x(k)=v(1);             % save each iterate
    y(k)=v(2);             % for plotting later
    for t=1:10             % use Newton's method for systems
        J=zeros(6,6);      % start with a zero Jacobian
        J(1,5)=-1;         % and fill in the nonzero elements
        J(2,2)=2*v(5);
        J(2,5)=2*v(2);
        J(2,6)=-1;
        J(3,3)=v(5);
        J(3,5)=v(3);
        J(4,4)=v(6);
        J(4,6)=v(4);
        J(5,1)=-1;
        J(5,2)=2*v(2);
        J(5,3)=1;
        J(6,2)=-1;
        J(6,4)=1;

        F=zeros(6,1);                % make F a column vector
        F(1)=1-v(5);                 % and fill in the function values
        F(2)=-2+2*v(5)*v(2)-v(6);
        F(3)=-mu+v(3)*v(5);
        F(4)=-mu+v(4)*v(6);
        F(5)=-v(1)+v(2)^2-1+v(3);
        F(6)=-v(2)+v(4);

        d=inv(J)*(-F);     % find the correction vector
        alpha=1;           % make sure s and lambda stay positive
        for j=3:6
            if(d(j) < 0) alpha=min(alpha,0.99999*(-v(j)/d(j))); end
        end
        v=v+alpha*d;       % take the restricted step
    end                    % Lagrange conditions solved for this mu
    mus(k)=mu;                       % remember mu
    xerr(k)=norm([v(1);v(2)]-xstar); % and the error in x
    kappa(k)=cond(J);                % and the condition of J
    mu=mu/2;               % decrease mu
end                        % and continue
v                          % write the answer

% plot the convergence trajectory
figure(1); set(gca,'FontSize',25)
hold on
axis([-2,2,-2,2],'square')
for p=1:101
    xp(p)=-1+3*0.01*(p-1); 
    ypp(p)=sqrt(1+xp(p));
    ypm(p)=-sqrt(1+xp(p));
end
plot(xp,ypp)  % second constraint upper branch
plot(xp,ypm)  % second constraint lower branch
plot(x,y,'o') % iterates
plot(x,y)     % connected by lines
hold off
print -deps -solid b1intrj.eps

% plot solution error
figure(2); set(gca,'FontSize',25)
hold on
axis([1e-16,1,1e-16,10])
loglog(mus,xerr)
hold off
print -deps -solid b1inerr.eps

% plot condition number of J
figure(3); set(gca,'FontSize',25)
hold on
axis([1e-16,1,10,22])
semilogx(mus,kappa)
hold off
print -deps -solid b1incnd.eps
