% b1inq.m: accelerated interior-point solution of b1

clear; clf
xstar=[0;1];

for method=1:2                        % try both strategies
    v=[-2;2;0;0;1;1];
    ks(1)=0;                          % starting point
    xerr(1)=norm([v(1);v(2)]-xstar);  % has this error
    mu=0.5;
    for k=1:52
        J=zeros(6,6);
        J(1,5)=-1;
        J(2,2)=2*v(5);
        J(2,5)=2*v(2);
        J(2,6)=-1;
        J(3,3)=v(5);
        J(3,5)=v(3);
        J(4,4)=v(6);
        J(4,6)=v(4);
        J(5,1)=-1;
        J(5,2)=2*v(2);
        J(5,3)=1;
        J(6,2)=-1;
        J(6,4)=1;

        F=zeros(6,1);
        F(1)=1-v(5);
        F(2)=-2+2*v(5)*v(2)-v(6);
        F(3)=-mu+v(3)*v(5);
        F(4)=-mu+v(4)*v(6);
        F(5)=-v(1)+v(2)^2-1+v(3);
        F(6)=-v(2)+v(4);

        if(method==1)
           sigma=0.99999;             % fixed fraction-to-boundary
        else
           phi=sqrt(F'*F);            % merit function
           if(k==1) mzero=phi; end    % remember first value
           phi=phi/mzero;             % and use it to normalize each value
           sigma=max(0.5,1-phi);      % contingent fraction-to-boundary
        end

        d=inv(J)*(-F);
        alpha=1;
        for j=3:6
            if(d(j) < 0) alpha=min(alpha,sigma*(-v(j)/d(j))); end
        end
        v=v+alpha*d;
        ks(k+1)=k;
        xerr(k+1)=norm([v(1);v(2)]-xstar);

        if(method==1)
           mu=mu/2;                   % fixed reduction in mu
        else
           mu=min(0.5*phi,phi^2);     % contingent reduction in mu
        end
    end
    figure(method)                    % plot each error curve
    axis([0,52,1e-16,1e1])
    set(gca,'FontSize',30)
    hold on
    semilogy(ks,xerr)
    hold off
    switch(method)
      case 1; print -deps -solid b1inl.eps
      case 2; print -deps -solid b1inq.eps
    end
end
