% b2bar.m: solve b2 by a sequence of barrier problems
clear;clf
format long

global prob='p2' m=1 mu=0                    % specify the problem
xl=[0;0]; xh=[3;3];                          % bounds for graph
xstar=[0.945582993415968;0.894127197437503]; % optimal point of p2
vc=[40,25,14,7,5,bta(xstar),1,.25];          % fix contour levels
figure(1); set(gca,'FontSize',25)            % first picture
hold on                                      % start plot
axis([xl(1),xh(1),xl(2),xh(2)],'square')     % scale graph axes
[xc,yc,zc]=gridcntr(@bta,xl,xh,200);         % grid b1 objective
contour(xc,yc,zc,vc)                         % plot the contours
for p=1:200                                  % compute
    xp(p)=2*(p-1)/(200-1);                   % points on
    yp(p)=xp(p)^2;                           % the equality
end                                          % constraint
plot(xp,yp)                                  % and plot them

xzero=[1;2];                                 % starting point
mu=20;                                       % starting multiplier
for k=1:55                                   % do the sequence 
    xk(k)=xzero(1);                          % for plotting later
    yk(k)=xzero(2);                          % save current point
    muk(k)=mu;                               % and current multiplier
    err(k)=norm(xstar-xzero);                % and solution error
    kappa(k)=cond(btah(xzero));              % and Hessian condition
    xbeta=ntfeas(xzero,10,1e-6,@p2,1);       % solve barrier problem
    xzero=xbeta;                             % start from there
    mu=mu/2;                                 % with lower multiplier
end                                          % end of sequence
xbeta                                        % report final point

plot(xk,yk,'o')                              % barrier solutions
plot(xk,yk)                                  % connected by lines
hold off                                     % done with plot
print -deps -solid b2bar.eps                 % print the plot
figure(2); set(gca,'FontSize',25)            % second picture
hold on                                      % start error plot
axis([1e-16,20,1e-16,1])                     % scale graph axes
loglog(muk,err)                              % log(err) vs log(mu)
hold off                                     % like log(err) vs k
print -deps -solid b2err.eps                 % print the plot
figure(3); set(gca,'FontSize',25)            % third picture
hold on                                      % start condition plot
axis([1e-16,20,1,1e18])                      % scale graph axes
loglog(muk,kappa)                            % log(kappa) vs log(mu)
hold off                                     % like log(kappa) vs k
print -deps -solid b2kappa.eps               % print the plot
