% bss1trust.m: study the first step in solving bss1
clear;clf
global r=0 g=zeros(2,1) H=zeros(2,2);
xl=[-2;0];
xh=[ 2;4];
xstar=[2;1];
xzero=[0;3];
diary

% find the first Newton descent step
H=bss1h(xzero);
[U,tp]=chol(H);
tp
g=bss1g(xzero);
y=U'\(-g);
d0=U\y;
nd0=norm(d0)

% step in that direction a distance r
r=0.5*norm(xstar-xzero) 
xnewt=xzero+r*(d0/norm(d0));
fnewt=bss1(xnewt)

% plot the trust-region error function
for t=1:101
    u=-100+0.01*(t-1)*140;
    xt(t)=u;
    yt(t)=truste(u);
end
figure(1); set(gca,'FontSize',30)
hold on
axis([-100,40,-1,2])
plot(xt,yt)
plot([-100;40],[0;0])
hold off
print -deps -solid bss1phi.eps

% solve the trust region subproblem exactly
uzero=[0,20];
ustar=fzero(@truste,uzero);
Hstar=H+ustar*eye(2);
eigs=eig(Hstar) 
[err,p]=truste(ustar);
xtrust=xzero+p;
ftrust=bss1(xtrust)

% plot the trust region over contours of q(x)
figure(2); set(gca,'FontSize',30)
hold on
axis([xl(1),xh(1),xl(2),xh(2)],"equal")
[xt,yt]=circle(xzero(1),xzero(2),r,101);
plot(xt,yt)
[xc,yc,zc,zmin,zmax]=gridcntr(@qbss1,xl,xh,50);
vc=[7,10,qbss1(xtrust),qbss1(xnewt),qbss1(xzero)];
contour(xc,yc,zc,vc)
plot([xzero(1);xnewt(1)],[xzero(2);xnewt(2)])
plot([xzero(1);xtrust(1)],[xzero(2);xtrust(2)])
hold off
print -deps -solid bss1q.eps

% plot the trust region over contours of f(x)
figure(3); set(gca,'FontSize',30)
hold on
axis([xl(1),xh(1),xl(2),xh(2)],'equal')
[xt,yt]=circle(xzero(1),xzero(2),r,101);
plot(xt,yt)
[xc,yc,zc,zmin,zmax]=gridcntr(@bss1,xl,xh,50);
vc=[0.05,0.25,2,5,bss1(xtrust),bss1(xnewt),bss1(xzero)];
contour(xc,yc,zc,vc)
plot([xzero(1);xnewt(1)],[xzero(2);xnewt(2)])
plot([xzero(1);xtrust(1)],[xzero(2);xtrust(2)])
hold off
print -deps -solid bss1f.eps

% solve the trust region subproblem approximately
[p,rcs]=dogsub(H,g,r);
xdog=xzero+p;
fdog=bss1(xdog);

% plot the trust region over contours of q(x)
figure(4); set(gca,'FontSize',30)
hold on
axis([xl(1),xh(1),xl(2),xh(2)],"equal")
[xt,yt]=circle(xzero(1),xzero(2),r,101);
plot(xt,yt)
[xc,yc,zc,zmin,zmax]=gridcntr(@qbss1,xl,xh,50);
vc=[7,10,qbss1(xtrust),qbss1(xnewt),qbss1(xzero)];
contour(xc,yc,zc,vc)
plot([xzero(1);xnewt(1)],[xzero(2);xnewt(2)])
plot([xzero(1);xtrust(1)],[xzero(2);xtrust(2)])
plot([xzero(1);xdog(1)],[xzero(2);xdog(2)])
hold off
print -deps -solid dogsubq.eps

% plot the trust region over contours of f(x)
figure(5); set(gca,'FontSize',30)
hold on
axis([xl(1),xh(1),xl(2),xh(2)],'equal')
[xt,yt]=circle(xzero(1),xzero(2),r,101);
plot(xt,yt)
[xc,yc,zc,zmin,zmax]=gridcntr(@bss1,xl,xh,50);
vc=[0.05,0.25,2,5,bss1(xtrust),bss1(xnewt),bss1(xzero)];
contour(xc,yc,zc,vc)
plot([xzero(1);xnewt(1)],[xzero(2);xnewt(2)])
plot([xzero(1);xtrust(1)],[xzero(2);xtrust(2)])
plot([xzero(1);xdog(1)],[xzero(2);xdog(2)])
hold off
print -deps -solid dogsubf.eps
