% cfysrun.m: classify using soft-margin SVM
clear; clf; set(gca,'FontSize',20)
global X Y c

% read and scale the enlarged problem data
load -ascii cfys.dat
X=(2/3)*cfys(:,1:2);
Y=cfys(:,3);

% use these values of the compromise parameter
cs(1)=0.1;
cs(2)=0.5;
cs(3)=1.0;
for k=4:49
  cs(k)=0.4*(k+1);
end

% solve the soft-margin SVM for the tabled values of c
printf('  c    intercepts   margin xi\n')
for k=1:49
  c=cs(k);                                      % set c
  pqxizero=zeros(11,1);                         % starting point
  pqxistar=sqp(pqxizero,@cfysobj,[],@cfysgeq);  % solve the NLP
  p=pqxistar(1:2);                              % extract p*
  xi=pqxistar(4:11);                            % extract xi*
  m(k)=2/norm(p);                               % save margin
  tce(k)=0;                                     % save
  for i=1:8                                     %   total
      tce(k)=tce(k)+xi(i);                      %   classification
  end                                           %   error

% print some of the results
  if(k <= 4 || k == 49)
     q=pqxistar(3);                             % extract q*
     x1=-q/p(1);                                % x1-intercept
     x2=-q/p(2);                                % x2-intercept
     printf('%5.1f  %5.2f %5.2f  %5.2f',c,x1,x2,m(k))
     for i=1:8
         printf('%6.2f',xi(i))
     end
     printf('\n')
  end
end

% plot error and margin as functions of c
hold on
plot(cs,tce)
plot(cs,m)
hold off
print -deps -solid cfys.eps
