function [h,rc,npt]=curve(name,mineq,muin,xstart,r,mxpt,dir)
% draw a single beta contour containing xstart

  global prob m mu    % prepare to use bta() and btag()
  prob=name;          % by filling in
  m=mineq;            % the global
  mu=muin;            % variables

  xp=xstart;          % start drawing a contour at xstart
  fcn=str2func(prob);      % pointer to function routine
  nofea=chkfea(xp,fcn,m);  % starting point feasible?
  if(nofea)                % if not then
     h=realmax;       % beta=infinity
     rc=8;            % signal failure
     npt=0;           % without drawing any points
     return           % and resign
  end                 % starting point is feasible
  h=bta(xp);          % it is on this contour
  xc=zeros(1,mxpt);   % initialize x coordinates of contour
  yc=zeros(1,mxpt);   % initialize y coordinates of contour
  left=0;             % the first search spans from 0
  right=dir*pi;       % to +180 degrees or -180 degrees
  tol=1e-6;           % set tolerance for finding the curve
  rc=0;               % assume we will succeed in drawing the contour
  closed=false;       % assume the contour will not be closed

  for p=1:mxpt        % find points on contour
      nozro=false;    % assume we will find this point
      nofea=false;    % assume the point will be feasible
      xc(p)=xp(1);    % x-coordinate to plot
      yc(p)=xp(2);    % y-coordinate to plot
      npt=p;          % number of points successfully found
      if(p > 2)                   % if far enough from start
         if(norm(xp-xstart) < r)  % check whether we have returned there
           closed=true;           % if so we have plotted a closed curve
           break                  % so this contour is done
         end          % otherwise we can continue
      end             % done checking for a closed curve
      g=btag(xp);                             % gradient at current point
      phi=-sign(g(2))*atan2(g(2),g(1));       % angle it is above x(1) axis
      al=left;                                % search from this angle
      xl=xp+[r*cos(al-phi);r*sin(al-phi)];    % which yields this point
      nofea=chkfea(xl,fcn,m);                 % is it feasible?
      if(nofea) break; end                    % if not give up
      fl=bta(xl)-h;                           % else it has this bta error
      ar=right;                               % search to this angle
      xr=xp+[r*cos(ar-phi);r*sin(ar-phi)];    % which yields this point
      nofea=chkfea(xr,fcn,m);                 % is it feasible?
      if(nofea) break; end                    % if not give up
      fr=bta(xr)-h;                           % else it has this bta error
      for t=1:52                              % do up to 52 bisections
          alpha=(al+ar)/2;                            % try the midpoint angle
          xa=xp+[r*cos(alpha-phi);r*sin(alpha-phi)];  % point at new angle
          nofea=chkfea(xa,fcn,m);                     % is it feasible?
          if(nofea) break; end                % if not give up
          if(norm(xr-xl) < tol)               % close enough?
             xp=xa;                           % yes; this is the root 
             break                            % save it to plot
          end                                 % done testing convergence
          f=bta(xa)-h;      % not done; find bta error at new root guess
          if(f*fl < 0)      % sign change from left to center?
             ar=alpha;      % yes
             xr=xa;         % move right end of interval to center
             fr=f;          % update that function value
             continue       % and keep bisecting
          end               % done testing
          if(f*fr < 0)      % sign change from center to right end?
             al=alpha;      % yes
             xl=xa;         % move left end of interval to center
             fl=f;          % update that function value
             continue       % and keep bisecting
          end               % done testing
          nozro=true        % no sign change; declare failure
          break             % and give up
      end                            % done accumulating points on contour
      if(nofea || nozro) break; end  % if no root was found contour is done
      left=alpha-pi/2;               % otherwise next search interval
      right=alpha+pi/2;     % is semicircle centered on this angle
  end                       % this contour is finished

  plot(xc(1:npt),yc(1:npt)) % plot the curve and report what happened
  if(nozro)  rc=rc+1; end   % bisection failed
  if(nofea)  rc=rc+2; end   % contour encountered boundary of S
  if(closed) rc=rc+4; end   % contour is a closed curve
end
