function [p,rcs]=dogsub(H,g,r)
% solve the trust-region subproblem approximately

  [U,pz]=chol(H);               % find dn, the full Newton step
  if(pz~=0)                     % H positive definite?
     rcs=1;                       % report failure
     return                       % and give up
  end                           % now Hd=U'Ud=-g
  y=U'\(-g);                    % solve U'y=-g for y
  dn=U\y;                       % solve Ud=y for dn

  if(norm(dn) <= r)             % inside trust region?
     p=dn;                      % yes; take full Newton step
  else                          % otherwise
     ds=-((g'*g)/(g'*H*g))*g;   % find steepest descent full step
     if(r <= norm(ds));         % on steepest-descent dogleg part
        tau=r/norm(ds);         % find where on the dogleg
        p=tau*ds;               % and the step to there
     else;                      % on connecting dogleg part
        a=(dn-ds)'*(dn-ds);     % find
        b=2*ds'*(dn-ds);        % coefficients
        c=ds'*ds-r^2;           % of quadratic

        tau=1+(-b+sqrt(b^2-4*a*c))/(2*a); % find where on dogleg

        p=ds+(tau-1)*(dn-ds);   % and the step to there
     end                        % finished finding dogleg part
  end                           % finished solving subproblem

  rcs=0;                        % report success
end
