function [xstar,rc,k,Qstar]=ea(xzero,Qzero,m,kmax,tol,fcn,grd)
% do up to kmax iterations of the ellipsoid algorithm to solve
% minimize fcn(x,0) subject to fcn(x,i) <= 0, i=1..m

% compute constants used in the updates
  n=size(xzero,1);
  a=1/(n+1);
  b=2*a;
  c=n^2/(n^2-1);

  x=xzero;
  Q=Qzero;
  rc=1;
  for k=1:kmax
%     find a function to use in making the cut
      icut=0;
      for i=1:m
          if(fcn(x,i) > 0)
             icut=i;
             break
          end
      end

%     find the gradient and normalize it
      g=grd(x,icut);
      ng=0;
      for j=1:n
          ng=max(ng,abs(g(j)));
      end
      if(ng == 0)             % gradient zero
         rc=3;
         break
      else
         g=g/ng;
      end

%     find the direction in which to move the ellipsoid center
      gqg=g'*Q*g;
      if(gqg <= 0)            % ellipsoid matrix not PD
         rc=2;
         break
      else
         d=-Q*g/sqrt(gqg);
      end

%     check for convergence
      xnew=x+a*d;
      if(norm(xnew-x) < tol)  % close enough
         rc=0;
         break
      else
         Qnew=c*(Q-b*d*d');
      end

%     update the ellipsoid for the next iteration
      x=xnew;
      Q=0.5*(Qnew+Qnew');
  end
  xstar=x;                    % done or out of iterations
  Qstar=Q;

end
