function [xstar,k,rc,lstar,pn,tstar]=emiqp(name,mi,xzero,kmax,epz)
% solve elastic mode penalty problem using iqp

  global prob m pn              % share these with em.m, emg.m, emh.m
    prob=name;                  % character name of original problem
    m=mi;                       % constraints in original problem
    pn=1;                       % starting penalty multiplier
  n=size(xzero,1);              % variables in original problem
  yk=[xzero;zeros(mi,1)];       % starting [x;0]
  fcn=str2func(prob);           % get function handle
  for i=1:mi
      yk(n+i)=max(0,fcn(xzero,i)); % initialize t(i) for feasibility
  end

  rc=1;
  for k=1:kmax
      [ystar,ki,rci,lambda]=iqp(@em,@emg,@emh,2*mi,yk,100,epz);
      if(rci > 2)
         rc=rci;
         break
      end

      if(norm(ystar-yk) < epz)  % close enough?
         rc=0;                  % signal success
         if(rci == 2) rc=2; end % or that multipliers not found
         break                  % and interrupt iterations
      else
         yk=ystar;              % start at current point
         pn=2*pn;               % double the penalty multiplier
      end
  end
  xstar=ystar(1:n);             % best x so far
  tstar=ystar(n+1:n+m);         % best t so far
  lstar=lambda(mi+1:2*mi);      % multipliers of original constraints

end
