C
Code by Michael Kupferschmid
C
C     This program filters "typescript" files to remove
C     control sequences.
C
C     variable  meaning
C     --------  -------
C     BEL       the bell character as an INTEGER*1
C     BELC      BEL as a CHARACTER*1
C     CTLH      ^H = hex[08] as an INTEGER*1
C     CTLHC     CTLH as a CHARACTER*1
C     CTLM      ^M = hex[0D] as an INTEGER*1
C     CTLMC     CTLM as a CHARACTER*1
C     ESC       the escape character as an INTEGER*1
C     ESCC      ESC as a CHARACTER*1
C     J         index on characters being blanked out
C     K         pointer to the character now being processed
C     KP        pointer into this ESC sequence
C     L         length of LINE
C     LENGTH    function returns index of last nonblank in a string
C     LINE      a line read in, cleaned up, and written out 
C     SHSTL     routine shifts a string left
C
      CHARACTER*1 LINE(1000),ESCC,BELC,CTLMC,CTLHC
      INTEGER*1 ESC/Z'1B'/,BEL/Z'07'/,CTLM/Z'0D'/,CTLH/Z'08'/
      EQUIVALENCE(ESCC,ESC)
      EQUIVALENCE(BELC,BEL)
      EQUIVALENCE(CTLMC,CTLM)
      EQUIVALENCE(CTLHC,CTLH)
C
C ------------------------------------------------------------------
C
C     process the next line
    9 READ(5,901,END=3) LINE
  901 FORMAT(1000A1)
      L=LENGTH(LINE,1000)
      K=0
    2 K=K+1
   10 IF(K.GT.L) GO TO 1
C
C        find the next escape sequence
         IF(LINE(K).EQ.ESCC) THEN 
            KP=1
C
C           is it esc-] ? 
            IF(K+KP.EQ.L) GO TO 4
            IF(LINE(K+KP).EQ.']') THEN
C              find the next ^G and delete thru it
    5          KP=KP+1
               IF(K+KP.EQ.L) GO TO 4
               IF(LINE(K+KP).NE.BELC) GO TO 5
               DO 6 J=K,K+KP
                    LINE(J)=' '
    6          CONTINUE
               CALL SHSTL(LINE(K),L,KP+1)
               L=L-KP
               GO TO 10 
            ENDIF
C
C           is it esc-[ ?
            IF(K+KP.EQ.L) GO TO 4
            IF(LINE(K+KP).EQ.'[') THEN
C              find the next m and delete thru it
    7          KP=KP+1
               IF(K+KP.EQ.L) GO TO 4
               IF(LINE(K+KP).NE.'m') GO TO 7
               DO 8 J=K,K+KP
                    LINE(J)=' ' 
    8          CONTINUE
               CALL SHSTL(LINE(K),L,KP+1)
               L=L-KP
               GO TO 10 
            ENDIF
C
    4    L=K
         ENDIF
C
C        check for backspaced corrections
         IF(K.GT.1 .AND. LINE(K).EQ.CTLHC) THEN
            LINE(K-1)=' '
            LINE(K)=' '
            CALL SHSTL(LINE(K-1),L,2)
            L=L-2
            K=K-1
            GO TO 10
         ENDIF
C
C        check for a carriage return
         IF(LINE(K).EQ.CTLMC) THEN
            LINE(K)=' '
            IF(K.EQ.L) THEN
C              it's the end of the line; go print it
               L=L-1
               GO TO 1 
            ELSE
C              there is more to this line
               L=L-1
            ENDIF
         ENDIF
      GO TO 2
C
C     done with the line
    1 WRITE(6,901) (LINE(K),K=1,L)
      GO TO 9
    3 STOP
      END
C
Code by Michael Kupferschmid
C
      SUBROUTINE SHSTL(LINE,L,N)
C     This routine shifts the characters in LINE left by N.
C
C     variable  meaning
C     --------  -------
C     K         index on the characters of LINE
C     L         length of LINE
C     LINE      string to be shifted left
C     MIN0      Fortran function for smaller of INTEGER*4s
C     N         number of characters LINE is to be shifted
C     NSHFT     actual amount of shift
C
      CHARACTER*1 LINE(*)
C
C ------------------------------------------------------------------
C
      IF(N.LE.0) RETURN
      NSHFT=MIN0(N,L)
      IF(NSHFT.LT.L) THEN 
         DO 1 K=1,L-NSHFT
              LINE(K)=LINE(K+NSHFT)
    1    CONTINUE
      ENDIF
    3 DO 2 K=L-NSHFT+1,L
           LINE(K)=' '
    2 CONTINUE
      RETURN
      END
C
Code by Michael Kupferschmid
C
      FUNCTION LENGTH(TEXT,LT)
C     This routine returns the index of the last nonblank in TEXT.
C
C     variable  meaning
C     --------  -------
C     K         index on the characters of TEXT
C     LT        length of TEXT
C     TEXT      the string whose length is to be found
C
      CHARACTER*1 TEXT(LT)
C
C ------------------------------------------------------------------
C
C     if there are no characters in the string its length is zero
      LENGTH=0
      IF(LT.LT.1) RETURN
C
C     examine characters right-to-left till finding first nonblank
      DO 1 K=1,LT
           LENGTH=LT-K+1
           IF(TEXT(LENGTH).NE.' ') RETURN
    1 CONTINUE
C     they were all blank
      LENGTH=0
      RETURN
      END
