function g=gradcd(fcn,x,ii)
% approximate the gradient of function ii by central differencing

  delta=4.80699951035563e-06;   % u^(1/3)
  n=size(x,1);                  % number of variables
  e=zeros(n,1);                 % e is a column of zeros
  g=zeros(n,1);                 % g is a column

  for j=1:n                     % for each coordinate direction
      e(j)=1;                   % make e the j'th unit vector
      xpd=x+delta*e;            % step forward by delta
      ypd=fcn(xpd,ii);          % find the function value there
      xmd=x-delta*e;            % step back by delta
      ymd=fcn(xmd,ii);          % find the function value there
      g(j)=(ypd-ymd)/(2*delta); % find approximation
      e(j)=0;                   % put e back to a zero vector
  end                           % done with the directions

end
