function [xstar,k,rc]=grg(fcn,grd,hsn,n,m,xzero,kmax,epz)
% minimize f(x) subject to F(x)=0.

  F=zeros(m,1);                   % declare sizes
  A=zeros(m,n); J=zeros(m,n-m);   % of built-up arrays
  xk=xzero;                       % feasible starting point
  for k=1:kmax                    % do up to kmax iterations
      for i=1:m                   % for each constraint
          g=grd(xk,i);            % find its gradient
          A(i,:)=g';              % construct its linear approximation
      end                         % constraint linearization ready
      Z=null(A);                  % get a basis for the nullspace
      R=orth(A');                 % get a basis for the range space

      g=grd(xk,0);                % objective gradient
      rg=Z'*g;                    % reduced gradient
      if(norm(rg) <= epz)         % converged?
         xstar=xk;                % yes; save optimal point
         rc=0;                    % report success
         return                   % and return
      end                         % done with convergence test
      rH=Z'*hsn(xk,0)*Z;          % reduced Hessian
      astar=(rg'*rg)/(rg'*rH*rg); % length of full steepest descent
      xsd=xk-Z*(astar*rg);        % take the step in R^n

      p=zeros(m,1);               % initialize correction step
      for s=1:20                  % Newton's method for systems
          xtry=xsd+R*p;                 % trial point
          for i=1:m                     % for each constraint
              F(i)=fcn(xtry,i);         % get function value
              J(i,:)=(R'*grd(xtry,i))'; % get del p value
          end                           % F and J updated for p
          delta=J\(-F);                 % correction
          p=p+delta;                    % update guess at p
          if(norm(delta) <= epz) break; end  % close enough?
      end                         % Newton's method done
      xk=xsd+R*p;                 % restore feasibility
  end                             % reduced gradient step done
  xstar=xk;                       % save the current point
  rc=1;                           % report out of iterations

end
