function h=hesscd(fcn,x,ii)
% approximate the Hessian of function ii by central differencing

  delta=1.02661016097495e-04;      % u^(1/4)
  n=size(x,1);                     % number of variables
  ei=zeros(n,1);                   % ei is a column of zeros
  ej=zeros(n,1);                   % ej is a column of zeros

  for j=1:n                        % for each column
      ej(j)=1;                     % make ej the j'th unit vector
      for i=j:n                    % for each row in lower triangle
          ei(i)=1;                 % make ei the i'th unit vector
          xpp=x+delta*ei+delta*ej; % ++ step 
          fpp=fcn(xpp,ii);         % function value
          xmp=x-delta*ei+delta*ej; % -+ step
          fmp=fcn(xmp,ii);         % function value
          xpm=x+delta*ei-delta*ej; % +- step
          fpm=fcn(xpm,ii);         % function value
          xmm=x-delta*ei-delta*ej; % -- step
          fmm=fcn(xmm,ii);         % function value

          h(i,j)=(fpp-fmp-fpm+fmm)/(4*delta^2); % find approximation

          h(j,i)=h(i,j);           % fill in the symmetric element
          ei(i)=0;                 % put ei back to a zero vector
      end                          % done with rows for this column
      ej(j)=0;                     % put ej back to a zero vector
  end                              % done with columns

end
