function [xstar,k,rc,lambda,mustar]=iqp(fcn,grd,hsn,m,xzero,kmax,epz)
% SQP algorithm for inequality-constrained problems

  n=size(xzero,1);               % variables
  xk=xzero;                      % starting point
  for i=1:m                      % consider each constraint
      mu(i)=0;                   % assume its multiplier is 0
      if(fcn(xk,i) > 0)          % but if xzero violates it
         mu(i)=1;                % make its multiplier 1
      end
  end
  A=zeros(m,n);                  % prepare A to be built up
  b=zeros(m,1);                  % prepare b to be built up
  rc=1;                          % anticipate nonconvergence
  for k=1:kmax                   % minimize the Lagrangian
      Q=hsn(xk,0);               % objective Hessian
      c=grd(xk,0);               % objective gradient
      for i=1:m                  % consider each constraint
          Q=Q+mu(i)*hsn(xk,i);   % find Lagrangian Hessian
          g=grd(xk,i);           % constraint gradient
          c=c+mu(i)*g;           % find Lagrangian gradient
          A(i,:)=g';             % linearize constraint
          b(i)=-fcn(xk,i);       % linearize constraint
      end                        % done preparing qp subproblem

    [p,kq,rcq,W,mu]=qpin(Q,c,A,b,50,1e-14); % solve subproblem
     if(rcq > 1)
        rc=rcq;
        return
     end

     xk=xk+p;                    % update xk
     if(norm(p) <= epz)          % close enough?
        rc=0;                    % signal success
        break                    % and return
     end                         % done testing convergence
  end;                           % Lagrange conditions solved
  xstar=xk;                      % return current iterate
  mustar=mu;                     % return current QP multipliers

% find multipliers corresponding to the original problem
  Abar=zeros(0,n);
  mbar=0;
  for i=1:m
      if(W(i) == 1)
         mbar=mbar+1;
         Abar(mbar,:)=grd(xk,i)';
      end
  end
  lambda=zeros(m,1);
  if(mbar > 0)
     g=grd(xk,0);
     [lambda,rc]=getlgm(m,Abar,W,g);
  end

end
