function [lambda,z]=mults(iact,x,grd)
% estimate KKT multipliers
% by minimizing the sum of absolute row deviations
% in the stationarity condition of the NLP

  n=size(x,1);                   % number of variables in NLP
  mact=size(iact,2);             % number of active constraints
  T=zeros(1+n,1+2*n+mact);       % the LP tableau is this big

  g0=grd(x,0);                   % the NLP objective gradient
  T(:,1)=[0;g0];                 % is the LP constant column

  for j=1:n                      % each LP d+ variable column
      T(:,1+j)=[1;zeros(n,1)];   % has cost coefficient 1
      T(1+j,1+j)=1;              % and +1 in constraint row j
  end

  for j=1:n                      % each LP d- variable column
      T(:,1+n+j)=[1;zeros(n,1)]; % has cost coefficient 1
      T(1+j,1+n+j)=-1;           % and -1 in constraint row j
  end

  for i=1:mact                   % each LP lambda variable column
      gi=grd(x,iact(i));         % has zero cost and
      T(:,1+2*n+i)=[0;-gi];      % negative constraint gradient
  end

  [dpdmla,rc,Tnew]=simplex(T,n,2*n+mact); % solve the LP
  lambda=dpdmla(2*n+1:2*n+mact);          % return the multipliers
  z=Tnew(1,1);                            % and the residual

end
