% newth35.m: restrict Newton step length to solve h35
clear;clf

xzero=[1;0.6];                         % starting point
xstar=[3;0.5];                         % catalog optimal point

xl=[ 0.0;0.0];                         % lower bounds for picture
xh=[15.0;2.0];                         % upper bounds for picture
ng=100; vc=[0.1,1,4,8,16,32,64,128];   % set contouring parameters
[xc,yc,zc]=gridcntr(@h35,xl,xh,ng);    % get function values on grid

epz=1e-6;                              % convergence tolerance
gama=0.5;                              % weight for modified Newton
for tr=1:2                             % try two step-restrictions
    if(tr == 1)                        % the first experiment
       r=norm(xstar-xzero);            % allows big steps
    else                               % the second
       r=0.001*norm(xstar-xzero);      % requires tiny steps
    end                                % finished setting r

    xk=zeros(1500); yk=zeros(1500);    % fix array sizes
    x=xzero;                           % starting point
    for k=1:1500;                      % do iterations
        xk(k)=x(1);                    % remember the point
        yk(k)=x(2);                    % for plotting later

        [xnewt,kp,nm,rc]=ntfs(x,1,epz,@h35g,@h35h,gama); % new point

        if(rc==0) break; end           % stop on zero gradient
        d=xnewt-x;                     % direction to move
        if(norm(d) < epz) break; end   % stop on short enough step
        s=min(r,norm(d));              % limit the steplength
        x=x+s*(d/norm(d));             % and move to the next xk
    end                                % done with iterations
    k                                  % report iterations used

    figure(tr); set(gca,'FontSize',30) % separate the plots
    hold on                            % begin plot
    axis([xl(1),xh(1),xl(2),xh(2)]);   % set axes
    contour(xc,yc,zc,vc)               % draw contour lines
    plot(xk(1:k),yk(1:k),'+');         % plot convergence trajectory
    plot(xk(1:k),yk(1:k));             % plot connecting lines
    hold off                           % done with plot
    if(tr == 1)                        % if big steps
       print -deps -solid nth35a.eps      % call the picture this
    else                               % if tiny steps
       print -deps -solid nth35b.eps      % call the picture this
    end                                % done printing the graph
end                                    % done with step-restrictions
