function [xstar,kp,nm,rc]=nt(xzero,xl,xh,kmax,epz,grd,hsn,gama)
% modified Newton using bisection line search
  n=size(xzero,1);                 % get number of variables
  xk=xzero;                        % set starting point
  nm=0;                            % no modifications yet
  rc=0;                            % assume it will converge
  tol=epz;                         % set line search tolerance
  for kp=1:kmax                    % do up to kmax descents
      g=grd(xk);                   % find uphill direction
      if(norm(g) <= epz)           % is xk stationary?
         xstar=xk;                   % yes; declare xk optimal
         rc=0;                       % flag convergence
         return                      % and return
      end                          % no; continue iterations
      H=hsn(xk);                   % get current Hessian matrix
      [U,p]=chol(H);               % try to factor it
      while(p ~= 0)                % does H need modification?
        if(gama >= 1 || gama < 0)  % yes; can it be modified?
           xstar=xk;                 % no; gama value prevents that
           rc=2;                     % flag nonconvergence
           return                    % and return
        end                        % yes; modification possible
        H=gama*H+(1-gama)*eye(n);    % average with identity
        [U,p]=chol(H);               % and try again
        nm=nm+1;                     % count the modification
      end                          % now Hd=U'Ud=-g
      y=U'\(-g);                   % solve U'y=-g for y
      dk=U\y;                      % solve Ud=y for d
      astar=bls(xk,dk,xl,xh,n,grd,tol);
      xk=xk+astar*dk;              % take the optimal step
  end                              % and continue
  xstar=xk;                        % out of iterations
  rc=1;                            % so no convergence yet
end
