function [xbeta,kp,rc,nr]=ntfeas(xzero,kmax,epz,fcn,m)
% interior-point plain Newton to minimize beta(x;mu)

  xk=xzero;                        % start from given point
  nr=0;                            % no steplength restrictions yet
  for kp=1:kmax                    % do up to kmax descent steps
      g=btag(xk);                  % gradient of beta
      if(norm(g) <= epz)           % close enough to stationary?
         xbeta=xk;                 % yes; take the current iterate
         rc=0;                     % flag convergence
         return                    % and return
      end                          % done checking convergence
      H=btah(xk);                  % Hessian of beta
      [U,p]=chol(H);               % factor it
      if(p ~= 0)                   % is it non-pd?
         xbeta=xk;                 % yes; take the current iterate
         rc=2;                     % flag nonconvergence
         return                    % and return
      end                          % done checking H pd
      y=U'\(-g);                   % solve for
      d=U\y;                       % full Newton step
      for t=1:52                   % make sure step stays in S
          xtry=xk+d;               % compute trial step
          ok=true;                 % assume xtry feasible
          for i=1:m                % check each inequality
              if(fcn(xtry,i) >= 0) % is constraint i violated?
                 ok=false;         % yes
              end                  % stepped outside of S
          end                      % done checking feasibility
          if(ok)                   % if xtry is feasible
             break                 % accept it
          else                     % otherwise
             d=d/2;                % decrease steplength
          end                      % and try again
      end                          % finished restricting step
      if(ok)                       % did we find one that works?
         xk=xtry;                  % yes; accept it
      else                         % otherwise
         xbeta=xk;                 % no Newton step stays in S
         rc=3;                     % flag nonconvergence
         return                    % and return
      end                          % the step is inside S
      if(t > 1) nr=nr+1; end       % count steplength restrictions
  end                              % continue Newton descent
  xbeta=xk;                        % take the current iterate
  rc=1;                            % and flag out of iterations

end
