function [xstar,k,rc,lstar]=ntlg(fcn,grd,hsn,n,m,xzero,lzero,kmax,epz)
% Newton-Lagrange algorithm for equality-constrained problems

  x=xzero;                          % starting point
  lambda=lzero;                     % starting multipliers
  rc=1;                             % in case of no convergence
  for k=1:kmax                      % do Newton's method for systems
      f=zeros(n+m,1);                 % fill in function vector
      f(1:n)=grd(x,0);                % gradient of objective
      for i=1:m                       % for each constraint
          lamg=lambda(i)*grd(x,i);    % weighted constraint gradient
          f(1:n)=f(1:n)+lamg;         % accumulate gradient of L
          f(n+i)=fcn(x,i);            % fill in function value
      end                             % done with f
      J=zeros(n+m,n+m);               % fill in Jacobian matrix
      J(1:n,1:n)=hsn(x,0);            % Hessian of objective
      for i=1:m                       % for each constraint
          lamH=lambda(i)*hsn(x,i);    % weighted constraint Hessian
          J(1:n,1:n)=J(1:n,1:n)+lamH; % accumulate Hessian of L
          J(1:n,n+i)=grd(x,i) ;       % fill in constraint gradient
          J(n+i,1:n)=grd(x,i)';       % and its transpose
      end                             % done with J
      delta=J\(-f);                   % find correction
      x=x+delta(1:n);                 % adjust x
      lambda=lambda+delta(n+1:n+m);   % adjust lambda
      if(norm(delta) <= epz)          % close enough?
         rc=0;                        % signal success
         break                        % and return
      end                             % done testing convergence
  end;                              % Lagrange conditions solved
  xstar=x;                          % return current iterate
  lstar=lambda;                     % and current multipliers

end
