function [xstar,kp,nm,rc,r]=ntrs(xzero,rzero,kmax,epz,fcn,grd,hsn,gama)
% adaptive modified Newton algorithm

  n=size(xzero,1);                 % get number of variables
  xk=xzero;                        % set starting point
  r=rzero;                         % set starting steplength
  mu=0.25; eta=0.75;               % set r adjustment parameters
  nm=0;                            % no Hessian modifications yet
  for kp=1:kmax                    % allow kmax descent iterations
      g=grd(xk);                   % find uphill direction
      if(norm(g) <= epz)           % is xk stationary?
         xstar=xk;                   % yes; declare xk optimal
         rc=0;                       % flag convergence
         return                      % and return
      end                          % no; continue iterations
      H=hsn(xk);                   % get current Hessian matrix
      [U,pz]=chol(H);              % try to factor it

      if(pz~=0)                    % is H positive definite?
        if(gama >= 1 || gama < 0)  % no; is modification possible?
           xstar=xk;                 % no; gama value prevents that
           rc=2;                     % flag nonconvergence
           return                    % and return
        end                        % yes; modification possible
        tmax=1022;                 % limit modifications
        for t=1:tmax               % repeat until limit or success
          H=gama*H+(1-gama)*eye(n);  % average with identity
          nm=nm+1;                   % count the modification
          [U,pt]=chol(H);            % try again to factor
          if(pt==0) break; end       % positive definite now?
        end                        % no; continue modifications
        if(pt~=0)                  % was modification successful?
          xstar=xk;                  % no; factorization still fails
          rc=3;                      % flag nonconvergence
          return                     % and return
        end                        % yes; modification succeeded
      end                          % now Hd=U'Ud=-g

      y=U'\(-g);                   % solve U'y=-g for y
      dk=U\y;                      % solve Ud=y for d
      if(xk+dk==xk)                % is the Newton step too small?
         xstar=xk;                 % yes; further descent impossible
         rc=4;                     % flag nonconvergence
         return                    % and return
      end                        % no; continue iterations

      if(kp==1)                    % start with rzero only if positive
         if(rzero <= 0) r=norm(dk); end  % else use full Newton step
      end                          % done initializing r
      tmax=52;                     % limit steplength adjustments
      for t=1:tmax                 % repeat until limit or success
          s=min(r,norm(dk));         % restrict steplength to r
          p=s*(dk/norm(dk));         % find trial step
          fxk=fcn(xk);                   % function value at xk
          gxk=grd(xk);                   % gradient at xk
          hxk=hsn(xk);                   % Hessian at xk
          qxtry=fxk+gxk'*p+0.5*p'*hxk*p; % quadratic model prediction
          xtry=xk+p;                     % find trial point
          fxtry=fcn(xtry);               % actual function value
          if(fxk==qxtry)                 % does the model go downhill?
             rho=(mu+eta)/2;               % no; any decrease is enough
          else                           % yes; continue adjustment
             rho=(fxk-fxtry)/(fxk-qxtry);  % reduction ratio
          end                            % done finding rho
          if(rho > mu)                   % enough reduction?
             xk=xtry;                    % yes; accept trial step
             if(rho >= eta) r=2*r; end   % increase r if possible
             break                       % and continue descent
          else                       % no, stepped too far
             r=r/2;                  % reduce steplength
             if(r == 0) break; end   % if process fails give up
          end                        % finished testing rho 
      end                            % finished adjusting r
      if(rho <= mu)                % was r adjustment successful?
         xstar=xtry;                 % no; return trial point
         rc=5;                       % flag nonconvergence
         return                      % and resign
      end                          % yes; r adjustment succeed
  end                              % continue descent

  xstar=xk;                        % out of iterations
  rc=1;                            % so no convergence yet
end
