% ntrsh35.m: solve h35 using ntrs.m
clear;clf

xzero=[1;0.6];                           % starting point
xstar=[3;0.5];                           % catalog optimal point

xl=[ 0.5;0.0];                           % lower bounds for picture
xh=[ 3.5;0.7];                           % upper bounds for picture
ng=100; vc=[0.1,0.5,1,2,3,4.5,6,8];      % set contouring parameters
[xc,yc,zc]=gridcntr(@h35,xl,xh,ng);      % get function values on grid

epz=1e-6;                                % convergence tolerance
gama=0.5;                                % weight for modified Newton
for tr=1:2
    if(tr == 1)                          % the first experiment
       r=norm(xstar-xzero);              % allows big steps
    else                                 % the second
       r=0.001*norm(xstar-xzero);        % requires tiny steps
    end                                  % finished setting r

    xk=zeros(1500); yk=zeros(1500);      % fix array sizes
    x=xzero;                             % starting point
    rzero=r;                             % starting steplength
    for k=1:20                           % do iterations
        xk(k)=x(1);                      % remember the point
        yk(k)=x(2);                      % for plotting later
        rk(k)=r;                         % remember the steplength
        kk(k)=k-1;                       % and iteration in which used

        [xstar,kp,nm,rc,r]=ntrs(x,rzero,1,epz,@h35,@h35g,@h35h,gama);

        if(rc==0) break; end             % stop on zero gradient
        x=xstar;                         % start next iteration
        rzero=r;                         % where this one left off
    end                                  % done with iterations
    k                                    % report iterations used

    figure(tr); set(gca,'FontSize',30)   % separate convergence plots
    hold on                              % begin plot
    axis([xl(1),xh(1),xl(2),xh(2)]);     % set axes
    contour(xc,yc,zc,vc)                 % draw contour lines
    plot(xk(1:k),yk(1:k),'+');           % plot convergence trajectory
    plot(xk(1:k),yk(1:k));               % plot connecting lines
    hold off                             % done with plot
    if(tr == 1)                          % if big steps
       print -deps -solid rsh35a.eps        % call the picture this
    else                                 % if tiny steps
       print -deps -solid rsh35b.eps        % call the picture this
    end                                  % done printing the graph
    figure(2+tr); set(gca,'FontSize',30) % separate steplength plots
    hold on                              % begin plot
    plot(kk,rk,'o')                      % plot r vs k
    plot(kk,rk)                          % plot connecting lines
    hold off                             % done with plot
    if(tr == 1)                          % if starting r
       print -deps -solid ntrsra.eps        % call the picture this
    else                                 % if tiny starting r
       print -deps -solid ntrsrb.eps        % call the picture this
    end                                  % done printing the graph
end                                      % done with experiments
