function [xstar,kp,nm,rc]=ntw(xzero,xl,xh,kmax,epz,fcn,grd,hsn,gama)
% modified Newton using Wolfe line search
  n=size(xzero,1);                 % get number of variables
  xk=xzero;                        % set starting point
  nm=0;                            % no modifications yet
  rc=0;                            % assume it will converge
  mu=0.0001;                       % Wolfe sufficient decrease
  eta=0.4;                         % Wolfe curvature
  smax=52;                         % line search iteration limit
  for kp=1:kmax                    % do up to kmax descents
      g=grd(xk);                   % find uphill direction
      if(norm(g) <= epz)           % is xk stationary?
         xstar=xk;                   % yes; declare xk optimal
         rc=0;                       % flag convergence
         return                      % and return
      end                          % no; continue iterations
      H=hsn(xk);                   % get current Hessian matrix
      [U,p]=chol(H);               % try to factor it
      while(p ~= 0)                % does H need modification?
        if(gama >= 1 || gama < 0)  % yes; can it be modified?
           xstar=xk;                 % no; gama value prevents that
           rc=2;                     % flag nonconvergence
           return                    % and return
        end                        % yes; modification possible
        H=gama*H+(1-gama)*eye(n);    % average with identity
        [U,p]=chol(H);               % and try again
        nm=nm+1;                     % count the modification
      end                          % now Hd=U'Ud=-g
      y=U'\(-g);                   % solve U'y=-g for y
      dk=U\y;                      % solve Ud=y for d
      tol=1000*epz*norm(g);        % adapt line search tolerance
      [astar,rcw,kw]=wolfe(xk,dk,xl,xh,n,fcn,grd,mu,eta,tol,smax);
      if(rcw > 3) break; end       % resign if line search failed
      xk=xk+astar*dk;              % take the optimal step
  end                              % and continue
  xstar=xk;                        % out of iterations
  rc=1;                            % so no convergence yet
end
