% p2pen.m: solve p2 by a sequence of penalty problems
clear;clf
format long

global prob='p2' m=1 mu=0                    % specify the problem
xl=[0;0]; xh=[3;3];                          % bounds for graph
xstar=[0.945582993415968;0.894127197437503]; % optimal point of p2
vc=[40,25,14,7,5,pye(xstar),1,.25];          % fix contour levels
figure(1); set(gca,'FontSize',30)            % first picture
axis([xl(1),xh(1),xl(2),xh(2)],'square')     % scale graph axes
hold on                                      % start plot
[xc,yc,zc]=gridcntr(@pye,xl,xh,200);         % grid p2 objective
contour(xc,yc,zc,vc)                         % plot the contours
for p=1:200                                  % compute
    xp(p)=2*(p-1)/(200-1);                   % points on
    yp(p)=xp(p)^2;                           % the equality
end                                          % constraint
plot(xp,yp)                                  % and plot them

xzero=[1;2];                                 % starting point
mu=0.05;                                     % starting multiplier
for k=1:59                                   % do the sequence 
    xk(k)=xzero(1);                          % for plotting later
    yk(k)=xzero(2);                          % save current point
    muk(k)=mu;                               % and current multiplier
    err(k)=norm(xstar-xzero);                % and solution error

    xpi=ntchol(xzero,10,1e-6,@pyeg,@pyeh);   % solve penalty problem
    xzero=xpi;                               % start from there
    mu=2*mu;                                 % with higher multiplier
end                                          % end of sequence
xpi                                          % report final point

plot(xk,yk,'o')                              % penalty solutions
plot(xk,yk)                                  % connected by lines
hold off                                     % done with plot
print -deps -solid p2pen.eps                 % print the plot
figure(2); set(gca,'FontSize',30)            % second picture
axis([0.05,1e16,1e-16,1])                    % scale graph axes
hold on                                      % start error plot
loglog(muk,err)                              % log(err) vs log(mu)
hold off                                     % is like log(err) vs k
print -deps -solid p2err.eps                 % print the plot
