% padmm.m: parallel ADMM with delayed updates
clear; format long; clf
global mu=1 A=zeros(2,2) lambda=ones(2,1) v=zeros(2,1)

xzero=[0;0;0;0];                             % unconstrained optimum
y1=xzero(1:2); y2=xzero(3:4);                % partition variables
A1=[3,-1;-4,1]; A2=[-2,-1;5,2]; b=[-1;3];    % partition constraints

x1k(1)=y1(1); x2k(1)=y1(2);                  % save y1 coordinates
delta=2*mu*(A1*y1+A2*y2-b);                  % feasiblity correction
ezero=delta'*delta;                          % starting error
err(1)=1; its(1)=0;                          % prepare to plot error

for k=1:1000    % do method-of-multiplier iterations
    v=A2*y2-b;  % constraint terms fixed while optimizing over y1
    A=A1;       % y1 partition of constraints

    y1new=ntrs(y1,0,10,1e-12,@admmf,@admmg,@admmh);

    v=A1*y1-b;  % constraint terms fixed while optimizing over y2
    A=A2;       % y2 partition of constraints

    y2new=ntrs(y2,0,10,1e-12,@admmf,@admmg,@admmh);

    delta=2*mu*(A1*y1+A2*y2-b); % feasibility correction
    lambda=lambda+delta;        % update lambda

    y1=y1new;   % wait to update y1 and y2
    y2=y2new;   % until the end of the iteration

    x1k(k+1)=y1(1); x2k(k+1)=y1(2);          % save y1 coordinates
    err(k+1)=delta'*delta/ezero; its(k+1)=k; % save error
end

xstar=[y1;y2] % report optimal point
lambda        % and optimal multipliers

figure(1)     % plot convergence
set(gca,'FontSize',25); hold on
axis([-0.8,0.4,-0.8,0.4],'square')
plot(x1k,x2k)
plot([0,0],[-0.8,0.4])
plot([-0.8,0.4],[0,0])
hold off
print -deps -solid padmmcnv.eps

figure(2)     % plot error curve
set(gca,'FontSize',25); hold on
axis([0,1000,1e-20,1e0],'square')
semilogy(its,err)
hold off
print -deps -solid padmmerr.eps
