function [Tnew,Snew,rc2]=phase2(T,S,mm,nn,tr,mr)
%  optimize a tableau in canonical form, or find it unbounded

   ztol=1e-6;                      % set zero tolerance
   Tnew=T;                         % return T on failure
   Snew=S;                         % return S on failure

   kmax=factorial(nn-1)/factorial(nn-mm); % theoretical maximum
   if(kmax > intmax-2) kmax=intmax-2; end % integer iteration limit

   for k=1:kmax                    % do up to kmax pivots
       cmin=0;                     % find
       jp=0;                       % in
       ii=tr(1);                   % the objective row
       for jj=2:nn                 % the column
           if(T(ii,jj) < cmin)     % with the lowest
              cmin=T(ii,jj);       % negative cost entry
              jp=jj;               % and remember the column number
           end                     % finish testing cost entry
       end                         % finish finding least cost entry

       if(jp == 0 || cmin > -ztol) % no (sufficiently) negative cost
          rc2=0;                   % signal optimal form
          return                   % and return to the caller
       end                         % finished testing for optimality

       ip=minr(T,tr,mr,jp);        % find min ratio row in column jp
       if(ip == 0)                 % if there is none
          rc2=jp;                  % signal unbounded in column jp
          return                   % and return to the caller
       end                         % finished finding pivot row

       [Tnew,Snew,rc]=pivot(T,mm,nn,ip,jp,S); % pivot at T(ip,jp)
       T=Tnew;                                % update the tableau
       S=Snew;                     % and the basic sequence
   end                             % for the next iteration
end
