function [xstar,kp,rc,nm]=qpeq(Q,c,A,b,kmax,epz)
% minimize (1/2)x'Qx+c'x subject to Ax=b

% size up the problem
  n=size(Q,1);                     % number of variables
  m=size(A,1);                     % number of equality constraints
  kp=0;                            % no iterations yet
  nm=0;                            % no modifications yet

% find a starting point and the inverse of the reduced Hessian
  xzero=zeros(n,1);                % use the origin if unconstrained
  if(m > 0)                        % if there are constraints
     T=[0,zeros(1,n);b,A];                              % tableau
     [Tnew,S,tr,mr,rc0]=newseq(T,m+1,n+1,[1:m+1],m+1);  % seek basis
     if(rc0 ~= 0)                                       % success?
        rc=3;                      % report constraints inconsistent
        return                     % and give up
     end
     for j=1:n                     % extract
         if(S(j) ~= 0)             % the basic solution
            xzero(j)=Tnew(S(j),1); % to use
         end                       % as the starting point
     end
     if(mr-1 == n)                 % is the system square?
        xstar=xzero;               % if so this is the optimal point
        rc=0;                      % report success
        return                     % and return it
     end
     A=Tnew(2:mr,2:n+1);           % A without redundant constraints
     Z=null(A);                    % get a basis for the nullspace
  else                             % no constraints
     Z=eye(n);                     % Z=I makes Z'*Q*Z=Q
  end
  [U,rch,nm]=hfact(Z'*Q*Z,0.5);    % factor the reduced Hessian
  if(rch ~= 0)                     % success?
     rc=2;                         % report modification failed
     return                        % and give up
  end
  V=Z/U;                           % solve VU=Z
  Hinv=V*V';                       % find Z*inv(Z'QZ)*Z'

% do modified Newton descent in the flat defined by the constraints
  xk=xzero;                        % start here
  for kp=1:kmax                    % do up to kmax iterations
      d=-Hinv*(Q*xk+c);            % full reduced Newton step
      if(norm(d) <= epz)           % converged?
         xstar=xk;                 % yes; save optimal point
         rc=0;                     % report success
         return                    % and return
      end
      xk=xk+d;                     % take the step
  end                              % of reduced Newton steps
  xstar=xk;                        % save the current point
  rc=1;                            % report out of iterations

end
