function [xstar,k,rc]=rsdeq(grd,hsn,A,b,kmax,epz)
% minimize f(x) subject to Ax=b

% size up the problem
  n=size(A,2);                     % number of variables
  m=size(A,1);                     % number of equality constraints
  k=0;                             % no iterations yet

% find a starting point and a basis for nullspace of A
  xzero=zeros(n,1);                % use the origin if unconstrained
  if(m > 0)                        % if there are constraints
     T=[0,zeros(1,n);b,A];                              % tableau
     [Tnew,S,tr,mr,rc0]=newseq(T,m+1,n+1,[1:m+1],m+1);  % seek basis
     if(rc0 ~= 0)                                       % success?
        rc=3;                      % report constraints inconsistent
        return                     % and give up
     end
     for j=1:n                     % extract
         if(S(j) ~= 0)             % the basic solution
            xzero(j)=Tnew(S(j),1); % to use
         end                       % as the starting point
     end
     if(mr-1 == n)                 % is the system square?
        xstar=xzero;               % if so this is the optimal point
        rc=0;                      % report success
        return                     % and return it
     end
     A=Tnew(2:mr,2:n+1);           % A without redundant constraints
     Z=null(A);                    % get a basis for the nullspace
  else                             % no constraints
     Z=eye(n);                     % Z=I does sd unconstrained
  end

% full-step steepest descent in the flat defined by the constraints
  xk=xzero;                        % start here
  for k=1:kmax                     % do up to kmax iterations
      rH=Z'*hsn(xk)*Z;             % Hessian in the flat
      rg=Z'*grd(xk);               % gradient in the flat
      astar=(rg'*rg)/(rg'*rH*rg);  % full step
      tk=Z'*(xk-xzero);            % current point in the flat
      tkp=tk+astar*(-rg);          % new point in the flat
      xk=Z*tkp+xzero;              % new point in R^n
      if(norm(rg) <= epz)          % converged?
         xstar=xk;                 % yes; save optimal point
         rc=0;                     % report success
         return                    % and return
      end
  end                              % of reduced Newton steps
  xstar=xk;                        % save the current point
  rc=1;                            % report out of iterations
end
