function [xstar,k,rc,lstar]=sqp(fcn,grd,hsn,n,m,xzero,lzero,kmax,epz)
% SQP algorithm for equality-constrained problems

  x=xzero;                        % starting point
  lambda=lzero;                   % starting multipliers
  A=zeros(m,n); b=zeros(m,1);     % prepare A and b to be built up
  rc=1;                           % in case of no convergence
  for k=1:kmax                    % minimize the Lagrangian
      Q=hsn(x,0);                 % objective Hessian
      c=grd(x,0);                 % objective gradient
      for i=1:m                   % for each constraint
          Q=Q+lambda(i)*hsn(x,i);   % find Lagrangian Hessian
          g=grd(x,i);               % constraint gradient
          c=c+lambda(i)*g;          % find Lagrangian gradient
          A(i,:)=g';                % linearize constraint
          b(i)=-fcn(x,i);           % linearize constraint
      end                         % done preparing qp subproblem

     [p,kq,rcq,nm]=qpeq(Q,c,A,b,50,1e-16); % solve the qp subproblem
      if(rcq > 1)
         rc=2;
         break
      end

      x=x+p;                      % update x
      [U,rch,nm]=hfact(A*A',1);   % factor
      Vt=A'/U;                    % and solve
      Aplus=(Vt/U')';             % to find the pseudoinverse
      mu=-Aplus*(Q*p+c);          % find the change in lambda
      lambda=lambda+mu;           % update lambda
      if(norm(p) <= epz)          % close enough?
         rc=0;                      % signal success
         break                      % and return
      end                           % done testing convergence
  end;                            % Lagrange conditions solved
  xstar=x;                        % return current iterate
  lstar=lambda;                   % and current multipliers
end
