% sqp1plot.m: graphical solution of sqp1
clear; clf; set(gca,'FontSize',15)
format long

xzero=[-1;1];                         % starting point
xk(1)=xzero(1);                       % save coordinates
yk(1)=xzero(2);                       % for plotting
x=xzero;                              % start solution there
lambda=1;                             % guess starting lambda
for k=1:10                            % Newton's method for systems
    f=[exp(x(1)-1)+2*lambda*x(1);     % update function vector
       exp(x(2)+1)+2*lambda*x(2);
       x(1)^2+x(2)^2-1];
    J=[exp(x(1)-1)+2*lambda,0,2*x(1); % update Jacobian
       0,exp(x(2)+1)+2*lambda,2*x(2);
       2*x(1),2*x(2),0];
    delta=J\(-f);                     % find correction
    x=x+delta(1:2);                   % update x part of solution
    xk(k+1)=x(1);                     % save coordinates
    yk(k+1)=x(2);                     % for ploting
    lambda=lambda+delta(3);           % update lambda of solution
end                                   % of Newton's method
xstar=x                               % report optimal point
lambda                                % report optimal lambda

xl=[-2.5;-2.5];                       % lower limits for plot
xh=[1.5;1.5];                         % upper limits for plot
ng=20;                                % grid points for contouring
[xc,yc,zc]=gridcntr(@sqp1c,xl,xh,ng); % function values on grid

hold on                                 % start graph
axis([xl(1),xh(1),xl(2),xh(2)],'equal') % set axes
v=[0.5,0.7,sqp1c(xstar)];               % contour levels
contour(xc,yc,zc,v)                   % contours of objective
for p=1:101                           % find points 
    x(p)=-1+2*0.01*(p-1);             % on zero contour
    yp(p)=+sqrt(1-x(p)^2);            % of the
    ym(p)=-sqrt(1-x(p)^2);            % constraint
end
plot(x,yp)                            % plot zero contour
plot(x,ym)                            % of the constraint
plot(xk,yk)                           % plot convergence trajectory
plot(xk,yk,'o')                       % mark the iterates
hold off                              % done with plot
print -deps -solid sqp1.eps           % print it
