% subopt.m: list all basic feasible solutions in objective order

% define the problem
T=[2325.0,0,0,18.750,76.250, 7.50,0,18.750;  % optimal tableau
      5.0,1,0, 2.750, 2.250, 0.50,0,-1.250;
      7.5,0,0, 1.625,-0.125, 0.25,1,-1.375;
     12.5,0,1,-1.125,-0.375,-0.25,0, 0.875];
S=[2,4,0,0,0,3,0];                           % its basic sequence
n=7;                                         % number of variables
m=3;                                         % number of functional constraints

maxpiv=factorial(n)/factorial(n-m)-1;        % only maxpiv pivots are possible
for npiv=1:maxpiv                            % so do no more than that
    T                                        % report the current tableau

    pos=0;                                   % count 
    for j=2:n+1                              % the
        if(T(1,j) > 0); pos=pos+1; end       % positive
    end                                      % costs
    if(pos == 0); break; end                 % pivot only until there are none left

%   find the next pivot away from optimality
    dzmin=realmax;
    jzmin=0;
    izmin=0;
    for j=1:n                                % examine each variable column
        if(S(j) == 0 && T(1,1+j) > 0)        % try nonbasic columns with positive cost

           rmin=realmax;
           for i=1:m                         % find
               if(T(1+i,1+j) > 0)            % the
                  r=T(i+1,1)/T(1+i,1+j);     % minimum
                  if(r < rmin)               % ratio
                     rmin=r;                 % pivot
                     imin=i;                 % position
                  end                        % in
               end                           % this
           end                               % column

%          pivoting there would increase the objective by this much
           dz=T(1+imin,1)*T(1,1+j)/T(1+imin,1+j);

           if(dz < dzmin)                    % we want
              dzmin=dz;                      % to change
              jzmin=j;                       % the objective
              izmin=imin;                    % as little
           end                               % as possible
        end
    end

%   perform the pivot yielding smallest dz
    mp=m+1;                                  % number of rows in tableau
    np=n+1;                                  % number of columns in tableau
    ip=izmin+1;                              % tableau row of pivot
    jp=jzmin+1;                              % tableau column of pivot
    [Tnew,Snew,rc]=pivot(T,mp,np,ip,jp,S);   % perform the pivot
    if(rc ~= 0); exit; end                   % quit if pivot failed
    T=Tnew;                                  % update the tableau
    S=Snew;                                  % update the basic sequence
end
