function [xstar,kp,rc]=trust(xzero,kmax,epz,fcn,grd,hsn)
% adaptive dogleg Newton algorithm

  x=xzero;                         % set starting point
  f=fcn(x);                        % construct
  g=grd(x);                        % quadratic
  H=hsn(x);                        % model

  [U,pz]=chol(H);                  % find dn, full Newton step
  if(pz~=0)                        % is H positive definite?
     rc=3;                         % no; report error
     return                        % and give up
  end                              % done checking factorization
  y=U'\(-g);                       % solve U'y=-g for y
  dn=U\y;                          % solve Ud=y for dn
  r=norm(dn);                      % its length is initial r
  mu=0.25; eta=0.75; tmax=52;      % set r adjustment parameters

  for kp=1:kmax                    % up to kmax optimization steps
    if(norm(g) <= epz)             % is x close to stationary?
       xstar=x;                      % yes; declare x optimal
       rc=0;                         % report convergence
       return                        % and return
    end                            % not done yet

    for t=1:tmax                   % find best p for a suitable r
      [p,rcs]=dogsub(H,g,r);       % p from trust region subproblem
      if(rcs~=0)                   % is H positive definite?
         r=r/2;                      % no; reduce r
         continue                    % and try again
      end                          % done checking subproblem
      xtry=x+p;                    % trial point
      qtry=f+g'*p+0.5*p'*H*p;      % quadratic model value there
      ftry=fcn(xtry);              % actual objective value there
      rho=(f-ftry)/(f-qtry);       % reduction ratio
      if(rho > mu)                 % accept trial step?
         if(rho >= eta) r=2*r; end   % yes; increase r if possible
         break                       % found suitable r and best p
      else                         % model is untrustworthy
         r=r/2;                      % reduce trust region radius
      end                          % finished testing trial step
    end                            % finished adjusting radius
    if(rho <= mu)                  % did radius adjustment succeed?
      rc=2;                          % no; report failure
      xstar=xtry;                    % return the trial point
      return                         % and give up
    end                            % finished checking success

    x=xtry;                        % move to the accepted point
    f=fcn(x);                      % update
    g=grd(x);                      % quadratic
    H=hsn(x);                      % model
  end                              % continue optimization steps
  rc=1;                            % report out of iterations
  xstar=x;                         % return the current solution

end
