% tryqn.m: compare DFP to BFGS on the rb problem
clear; clf; set(gca,'FontSize',25)

xl=[-2;-1];
xh=[2;2];
xzero=[-1.2;1];
xstar=[1;1];
ezero=norm(xzero-xstar);
kmax=100;
epz=1e-9;

% solve the problem using DFP
xk=xzero;
Gk=eye(2);
for kp=1:kmax
    x=xk;
    G=Gk;
    [xk,Gk,kused,rc]=dfp(x,G,xl,xh,1,epz,@rb,@rbg);
    kdfp(kp)=kp;
    edfp(kp)=norm(xk-xstar)/ezero;
    if(edfp(kp) < epz) break; end
end
printf('DFP:  x= %17.15f %17.15f at kp=%i3\n',xk(1),xk(2),kp)

% solve the problem using BFGS
xk=xzero;
Gk=eye(2);
for kp=1:kmax
    x=xk;
    G=Gk;
    [xk,Gk,kused,rc]=bfgs(x,G,xl,xh,1,epz,@rb,@rbg);
    kbfgs(kp)=kp;
    ebfgs(kp)=norm(xk-xstar)/ezero;
    if(ebfgs(kp) < epz) break; end
end
printf('BFGS: x= %17.15f %17.15f at kp=%i3\n',xk(1),xk(2),kp)

% plot error versus iteration for the two methods
hold on
semilogy(kdfp,edfp);
semilogy(kdfp,edfp,'+');
semilogy(kbfgs,ebfgs);
semilogy(kbfgs,ebfgs,'o');
hold off
print -deps -solid tryqn.eps
