function [xlr,xr,xhr,rc,k]=wander(xl,xh,m,kmax,tol,fcn,grd)
% do up to kmax iterations of the recentering ellipsoid algorithm
% to minimize fcn(x,0) subject to fcn(x,i) <= 0, i=1..m

% compute constants used in the updates
  n=size(xl,1);
  a=1/(n+1);
  b=2*a;
  c=n^2/(n^2-1);
  shr=0.1*(1-1/(2*n));

% initialize the ellipsoid center and matrix
  [x,Q]=eainit(xl,xh);

  rc=1;
  fr=realmax;
  for k=1:kmax
%     find a function to use in making the cut
      icut=0;
      for i=1:m
          if(fcn(x,i) > 0)
             icut=i;
             break
          end
      end

%     update the record point
      if(icut == 0)
         fobj=fcn(x,0);
         if(fobj < fr)
            fr=fobj;
            xr=x;
         end
      end

%     find the gradient and normalize it
      g=grd(x,icut);
      ng=0;
      for j=1:n
          ng=max(ng,abs(g(j)));
      end
      if(ng == 0)
         rc=3;
         break
      else
         g=g/ng;
      end

%     recenter or take the next step
      gqg=g'*Q*g;
      if(gqg <= 0)                        % is Q non-pd?
         if(norm(xh-xl) < tol)            % yes; xr close enough?
            rc=0;                         % yes; flag convergence
            break                         % and return
         else                             % not close enough
            if(fr < realmax)              % know a record point?
               for j=1:n                  % yes
                   w=shr*(xh(j)-xl(j));   % new bound width
                   xl(j)=xr(j)-0.5*w;     % new lower bound
                   xh(j)=xr(j)+0.5*w;     % new upper bound
               end                        % bounds now recentered
               [xnew,Qnew]=eainit(xl,xh); % find a new ellipsoid
            else                          % no record point
               rc=2;                      % can't recenter
               break                      % so give up
            end
         end
      else                                % Q is still pd
        d=-Q*g/sqrt(gqg);                 % find direction vector
        xnew=x+a*d;                       % find next center
        Qnew=c*(Q-b*d*d');                % and ellipsoid matrix
      end

%     update the ellipsoid for the next iteration
      x=xnew;
      Q=0.5*(Qnew+Qnew');
  end
  xlr=xl;
  xhr=xh;

end
